package de.communardo.confluence.plugins.createsubspace;

import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;

import com.atlassian.confluence.security.Permission;
import com.atlassian.confluence.security.PermissionManager;
import com.atlassian.confluence.security.SpacePermission;
import com.atlassian.confluence.setup.settings.Settings;
import com.atlassian.confluence.spaces.Space;
import com.atlassian.confluence.spaces.actions.CreateSpaceAction;

/**
 * An action for space creation.
 * 
 * @author Bill Martin
 * 
 */
public class CreateSpaceHierarchyAction extends CreateSpaceAction {

    private static final long serialVersionUID = 7215641065103630518L;

    /** Logger. */
    private static final Logger LOG = Logger.getLogger(CreateSpaceHierarchyAction.class);

    public static final String SPACE_KEY_DASHBOARD = "dashboard";

    /**
     * The child space. Null if there is no child space.
     */
    private Space childSpace = null;
    /**
     * The key of the child space. Null if there is no child space.
     */
    private String childSpaceKey = null;

    /**
     * {@inheritDoc}
     * 
     * @see com.atlassian.confluence.spaces.actions.CreateSpaceAction#execute()
     */
    public String execute() throws Exception {
        switchSpaceProperties();
        super.execute();
        switchSpaceProperties();

        String res = SUCCESS;
        boolean succ = CreateSubSpaceUtils.assignParentSpace(this.getSpace(), this.getChildSpace());

        if (!succ) {
            this.doRollback();
            res = ERROR;
        }
        return res;
    }

    /**
     * Switches the properties identifying the space for parent and child.
     */
    private void switchSpaceProperties() {
        String tmpKey = this.key;
        this.key = this.childSpaceKey;
        this.childSpaceKey = tmpKey;

        Space tmpSpc = this.space;
        this.space = this.childSpace;
        this.childSpace = tmpSpc;
    }

    /**
     * had to be overridden
     */
    public String spaceKeyAvailable() {
        switchSpaceProperties();
        String res = super.spaceKeyAvailable();
        switchSpaceProperties();

        return res;
    }

    private boolean isValidChildSpaceKey() {
        if (StringUtils.equalsIgnoreCase(this.getChildSpaceKey(), SPACE_KEY_DASHBOARD)) {
            return false;
        } else {
            switchSpaceProperties();
            boolean valid = super.isValidSpaceKey();
            switchSpaceProperties();
            return valid;
        }
    }

    /**
     * had to be overridden
     */
    public void validate() {
        switchSpaceProperties();
        super.validate();
        switchSpaceProperties();
        // validate fields for altered form
        if (!isValidChildSpaceKey()) {
            addFieldError("childSpaceKey", getText("space.key.invalid"));
        }
        if (spaceManager.getSpace(this.getChildSpaceKey()) != null) {
            addFieldError("childSpaceKey", getText("space.key.exists"));
        }
    }

    public Space getChildSpace() {
        String spcKey = this.getChildSpaceKey();
        if (this.childSpace == null && spcKey != null) {
            this.childSpace = this.spaceManager.getSpace(spcKey);
        }
        return this.childSpace;
    }

    /**
     * Gets the child key.
     */
    public String getChildSpaceKey() {
        return this.childSpaceKey;
    }

    /**
     * Sets the child key. Will be parameterized with childSpaceKey from form.
     * 
     * @param spaceKey
     */
    public void setChildSpaceKey(String spaceKey) {
        this.childSpaceKey = spaceKey;
    }

    /**
     * Rolls back any changes. Useful if additional actions after space creation failed.
     */
    protected void doRollback() {
        // remove previously created space
        Space spc = this.getChildSpace();
        this.spaceManager.removeSpace(spc);
    }

    /**
     * Removes create space permission from required permission set.
     * 
     * @return the set of required permissions to change the parent space
     */
    public List getPermissionTypes() {
        List permissionTypes = super.getPermissionTypes();
        if (getSpace() != null) { // not on dashboard
            permissionTypes.remove(SpacePermission.CREATE_SPACE_PERMISSION);
        }
        return permissionTypes;
    }

    public Settings getSettings() {
        return this.settingsManager.getGlobalSettings();
    }

    /**
     * Checks if current user has administrative permissions on space.
     * 
     * @return true if current user has administrative permissions on space
     */
    public boolean isPermitted() {
        boolean perm = false;
        if (super.isPermitted()) {
            Space spc = getSpace();
            if (spc == null) { // on dashboard
                perm = permissionManager.hasPermission(getRemoteUser(), Permission.ADMINISTER,
                        PermissionManager.TARGET_APPLICATION);
            } else {
                perm = permissionManager.hasPermission(getRemoteUser(), Permission.ADMINISTER, spc);
            }
        }
        return perm;
    }
}
